#include <stdint.h>
#include "resources.h"

/*
 * Ozone CS pin
 ******************************************************************************/
sbit OZONE_CS at LATC2_bit;
sbit OZONE_CS_DIR at TRISC2_bit;

/*
 * Global vars
 ******************************************************************************/
const float    Rr    = 10000.0;
const float    Vref  = 5.0;
uint16_t       raw;
float          ozone;
char           text[ 20 ];

/*
 * Prototypes
 ******************************************************************************/
void system_init( void );
void display_result( void );
void measurement( void );
void calculation( void );

/*
 * Functions
 ******************************************************************************/
void system_init()
{
    OZONE_CS_DIR = 0;
    Delay_ms( 200 );

    ANSELA = 0x00;
    ANSELB = 0x00;
    ANSELC = 0x00;
    ANSELD = 0x00;
    ANSELE = 0x00;
    ANSELG = 0x00;

    PPS_Mapping( 104, _OUTPUT, _SDO3 );
    PPS_Mapping( 98, _INPUT, _SDI3 );
    PPS_Mapping( 79, _OUTPUT, _SCK3OUT );
    SPI3_Init_Advanced( _SPI_MASTER, _SPI_8_BIT,
                        _SPI_PRESCALE_SEC_4, _SPI_PRESCALE_PRI_16,
                        _SPI_SS_DISABLE, _SPI_DATA_SAMPLE_END,
                        _SPI_CLK_IDLE_HIGH, _SPI_ACTIVE_2_IDLE );
    Delay_ms( 200 );
    
    PPS_Mapping( 100, _INPUT, _U1RX );
    PPS_Mapping( 101, _OUTPUT, _U1TX );
    UART1_Init( 9600 );
    Delay_ms( 200 );

    OZONE_CS = 1;
}


void display_result()
{
    FloatToStr( ozone, text );
    UART1_Write_Text( text );
    UART1_Write_Text( "\r\n" );
}

void measurement()
{
    uint8_t msb;
    uint8_t lsb;

    OZONE_CS = 0;
    msb = SPI_Rd_Ptr( 0xFF );
    lsb = SPI_Rd_Ptr( 0xFF );
    OZONE_CS = 1;

    raw = ( msb & 0x1F );
    raw <<= 7;
    raw |= ( lsb >> 1 );
}

void calculation()
{
    float Vout;
    float Rs;
    float ratio;
    float lgPPM;

    Vout    = ( float )raw / 4096.0 * Vref;
    FloatToStr( Vout, text );
    Rs = Rr / ( Vref - Vout ) * Vout;
    FloatToStr( Rs, text );
    ratio   = Rs / Rr;
    ozone   = ( ( log10( ratio ) *  -0.8 ) + 0.9 ) * 1000;  // Parts per billion
}

/*
 * Main
 ******************************************************************************/
void main()
{
    system_init();

    while( 1 )
    {
        measurement();
        calculation();
        display_result();
        delay_ms( 500 );
    }
}